﻿/// <reference path="JsDiagram-vsdoc.js" />
var DiagramView = MindFusion.Diagramming.DiagramView;
var Events = MindFusion.Diagramming.Events;
var Diagram = MindFusion.Diagramming.Diagram;
var DiagramLink = MindFusion.Diagramming.DiagramLink;
var LinkShape = MindFusion.Diagramming.LinkShape;
var ShapeNode = MindFusion.Diagramming.ShapeNode;
var AnchorPattern = MindFusion.Diagramming.AnchorPattern;
var AnchorPoint = MindFusion.Diagramming.AnchorPoint;
var MarkStyle = MindFusion.Diagramming.MarkStyle;
var GlassEffect = MindFusion.Diagramming.GlassEffect;
var Style = MindFusion.Diagramming.Style;
var Theme = MindFusion.Diagramming.Theme;
var Point = MindFusion.Drawing.Point;

var Rect = MindFusion.Drawing.Rect;

var diagram = null;
var bounds = new Rect(0, 0, 30, 18);

var d = MindFusion.Diagramming;
var counter = 0;
var colors = ["#F2D091", "#027373", "#0D2340", "#D12406", "#2A3E59", "#F25F29", "#65768C", "#e7a732", "#9FADBF", "#1FA88C"];

function getColor(index)
{
	while(index >= colors.length)
		index -= colors.length;
	
	return colors[index];
}

document.addEventListener("DOMContentLoaded", function ()
{
	document.getElementById("pullApartLinks").checked = false;	
	document.getElementById("layoutType").selectedIndex = 1;
	
	// create a Diagram component that wraps the "diagram" canvas
	var diagramView = DiagramView.create(document.getElementById("diagram_canvas"));
	diagram = diagramView.diagram;
	diagram.bounds = new Rect(5, 5, 2000, 1000);
	
	// register event handlers
    diagram.addEventListener(Events.nodeCreated, onNodeCreated);
	diagram.addEventListener(Events.nodeCreating, onNodeCreating);
	diagram.addEventListener(Events.linkCreated, onLinkCreated);
	diagram.addEventListener(Events.linkCreating, onLinkCreating);
    
    diagram.defaultShape = "Rectangle";	
	diagram.linkShape = d.LinkShape.Polyline;
	diagram.allowInplaceEdit = true;

	diagram.routeLinks = false;
	diagram.roundedLinks = false;
    diagram.showGrid = false;
	
    diagram.backBrush = "#f0f2f5";
	diagram.linkHeadShapeSize = 2;
	diagram.roundedLinksRadius = 35;
	
    var theme = new Theme();
    var shapeNodeStyle = new Style();
    shapeNodeStyle.brush = { type: 'LinearGradientBrush', color1: '#dad8d7', color2: '#a6a09f', angle: 90 };
    shapeNodeStyle.stroke = "#918988";
    shapeNodeStyle.textColor = "#585A5C";
    shapeNodeStyle.fontName = "Verdana";
    shapeNodeStyle.fontSize = 4;
    shapeNodeStyle.backBrush = "#e0e9e9"; 
    theme.styles.set("std:ShapeNode", shapeNodeStyle);
	
	var linkStyle = new Style();
    linkStyle.brush = { type: 'SolidBrush', color: '#0D2340'};
    linkStyle.stroke = "#0D2340";
    linkStyle.textColor = "#585A5C";
    linkStyle.fontName = "Verdana";
    linkStyle.fontSize = 3;
    linkStyle.backBrush = "#0D2340"; 
    theme.styles.set("std:DiagramLink", linkStyle);
	diagram.theme = theme;
   
	//create the nodes
	for(var i = 0; i < 10; i++)
	{
		
		var shape = diagram.factory.createShapeNode(bounds);
		
	}
	
	var link;
	
    //create random links
	diagram.nodes.forEach(function (node) {

		var nodeIndex = Math.floor(Math.random() * 10);

		var node2 = diagram.nodes[nodeIndex];
		link = diagram.factory.createDiagramLink(node, node2);
		link.headShape = "Circle";

	});  
	
	var node1 = diagram.nodes[3];
	var node2 = diagram.nodes[4];
	var links = [];
	for(var i = 0; i < 10; i++)
	{
		if(i % 2 == 0)
			link = diagram.factory.createDiagramLink(node1, node2);
		else
			link = diagram.factory.createDiagramLink(node2, node1);
			
		link.headShape = "Circle";
		link.text = "label" + (i+1).toString();
		link.textStyle = d.LinkTextStyle.MiddleSegmentRotated;
		links.push(link);
			
	}
	
	
    //apply layered layout
	var layout = new MindFusion.Graphs.LayeredLayout();
	layout.direction = MindFusion.Graphs.LayoutDirection.LeftToRight;
	layout.siftingRounds = 5;
	layout.nodeDistance = 10;
	layout.layerDistance = 20;
	diagram.arrange(layout);
	
	pullApartAsCurves(links);
	diagram.resizeToFitItems();	
	
	
});

	

//apply the chosen layout
function applyLayout()
{
	diagram.links.forEach(function (link) {

		link.SegmentCount = 1;

	});  	
	
	var x = document.getElementById("layoutType");
    if(x.selectedIndex == 0)
	{
		var layout = new MindFusion.Graphs.LayeredLayout();
		layout.direction = MindFusion.Graphs.LayoutDirection.LeftToRight;
		layout.siftingRounds = 5;
		layout.nodeDistance = 8;
		layout.layerDistance = 8;
		diagram.arrange(layout);
	}
	else if (x.selectedIndex == 1)
	{
		var flowchartLayout = new MindFusion.Graphs.FlowchartLayout();
		flowchartLayout.orientation = MindFusion.Graphs.Orientation.Horizontal;
		diagram.arrange(flowchartLayout);
	}
	
	pullApartAsCurves();
	diagram.resizeToFitItems();	

}

//change the color of newly created links while
//they are being drawn
function onLinkCreating(sender, args)
{
	var dlink = args.link;
	
	var commonLinks = getCommonLinks(
		dlink.origin, dlink.destination);
	var lcolor = getColor(commonLinks.length);
	
	dlink.stroke = lcolor;
	dlink.brush = { type: 'SolidBrush', color: lcolor};
	dlink.headShape = "Circle";
}

//change the color of newly created links
function onLinkCreated(sender, args)
{
	var dlink = args.link;
	
	var commonLinks = getCommonLinks(
		dlink.origin, dlink.destination);
	var lcolor = getColor(commonLinks.length);	
	dlink.stroke = lcolor;
	dlink.textColor = lcolor;
	dlink.textStyle = d.LinkTextStyle.MiddleSegmentRotated;
	dlink.brush = { type: 'SolidBrush', color: lcolor};
	dlink.headShape = "Circle";

	
	if (document.getElementById("pullApartLinks").checked)
	{
		
		if (commonLinks.length > 1)
			pullApartAsCurves(commonLinks);
	}
	
}

function pullApartAsCurves()
{
	let processed = new Map();
	
	for (let link of diagram.links) {	
        
	    if (processed.has(link))
			continue;
		
		let commonLinks = getCommonLinks(
			link.origin, link.destination);
			
		if (commonLinks.length > 1)
		{
			pullApartAsCurves(commonLinks);
			
			commonLinks.forEach((clink) => {
					processed.set(clink, true);
		    })
		}
	}	
}

function pullApartAsCurves(repeatingLinks)
{
	let even = repeatingLinks.length % 2 == 0;
	//the padding determines the distance between
	//adjacent arrows
	let padding = 8;

	let firstLink = repeatingLinks[0];
	resetStraight(firstLink);

	// for comparing link directions
	let firstOrigin = firstLink.origin;
	let firstDestination = firstLink.destination;

	// offset intermediate points away from these 
	let points = firstLink.controlPoints;
	let l = points.length - 1; //length?
	let pt1 = points[0];
	let pt2 = points[1];

	for (let i = 0; i < repeatingLinks.length; ++i)
	{
		if (!even && i == 0)
			continue;

	    let link = repeatingLinks[i];
	    resetStraight(link);

	    // make it a curve
	    link.shape = LinkShape.Bezier;
	    link.segmentCount = 1;

	    // offset control points, these are relative to first link
	    let cp1 = new Point(
	    	pt1.x + 1 * (pt2.x - pt1.x) / 3,
	    	pt1.y + 1 * (pt2.y - pt1.y) / 3);
	    let cp2 = new Point(
	    	pt1.x + 2 * (pt2.x - pt1.x) / 3,
	    	pt1.y + 2 * (pt2.y - pt1.y) / 3);

	    let radialCoords = MindFusion.Drawing.Geometry.cartesianToPolar(pt1, pt2);
	    let angle = radialCoords.a;
	    let pairOffset = even ? (1 + i / 2) * padding : ((i + 1) / 2) * padding;

    	cp1 = MindFusion.Drawing.Geometry.polarToCartesian(
    		cp1,  i % 2 == 0 ? angle - (Math.PI/2) : angle + (Math.PI/2), pairOffset);
    	cp2 = MindFusion.Drawing.Geometry.polarToCartesian(
    		cp2, i % 2 == 0 ? angle - (Math.PI/2) : angle + (Math.PI/2), pairOffset);

    	// assign points according to direction
		if (link.destination == firstDestination)
		{
			link.controlPoints[1] = new Point(cp1.x, cp1.y);
			link.controlPoints[2] = new Point(cp2.x, cp2.y);
		}
		else
		{
			link.controlPoints[1] = new Point(cp2.x, cp2.y);
			link.controlPoints[2] = new Point(cp1.x, cp1.y);
		}
		
		link.updateFromPoints();
	}
}

function resetStraight(link)
{
	// make link point to node center points
	link.shape = LinkShape.Polyline;
	link.segmentCount = 1;
	link.updateIntersections();
}

//get the links that bind the same nodes no matter
//the direction
function getCommonLinks(node1, node2)
{ 
	var commonLinks = [];

	node1.outgoingLinks.forEach((element) => {
        
	if (element.destination == node2)
	    commonLinks.push(element);
	})
	
	node1.incomingLinks.forEach((element) => {
        
	if (element.origin == node2)
	    commonLinks.push(element);
	})

	return commonLinks;
}



//apply custom anchor pattern, if checked
function onNodeCreated(sender, args)
{
	var node = args.node;
    node.Brush = { type: 'LinearGradientBrush', color1: '#c2bdbd', color2: '#5d5656', angle: 90 };
}

//change the color of new nodes that are being created now
function onNodeCreating(sender, args)
{
	var node = args.node;
	node.Brush = { type: 'LinearGradientBrush', color1: '#c2bdbd', color2: '#5d5656', angle: 90 };
}




